#!/bin/bash
# 仅仅在nfsdesktop 5.0系统下执行

PATH=/bin:/sbin:/usr/bin:/usr/sbin:/usr/local/bin:/usr/local/sbin:~/bin
export PATH
LANG=en_US.UTF-8

SERVER_IP=$1
DOMAIN_NAME=$2
domain_config=/usr/share/nfs-domain-client/salt/config.ini
repo_source=/etc/apt/sources.list.d/${SERVER_IP}.list

OSType=$(cat /etc/os-release | grep "^ID=" | awk -F '=' '{print $2}')
OSVersionId=$(cat /etc/os-release | grep "^VERSION_ID=" | awk -F '=' '{print $2}')

echo "服务器IP地址: $SERVER_IP, 域服务器地址：$DOMAIN_NAME, 操作系统: ${OSType}, 版本： ${OSVersionId}"

if [ "${OSType}" != 'nfsdesktop' ] || [ "${OSVersionId}" != '"5.0"' ]; then
	echo "Error: must be nfsdesktop 5.0 OS"
	exit 1
fi

Set_HOSTS(){
	hostcheck=$(cat /etc/hosts|grep -c "^${SERVER_IP} ${DOMAIN_NAME}")
	if [ "${hostcheck}" == '1' ];then
		return
	fi

	# 重新写入IP和域名信息
	sed -i "/${SERVER_IP}/d" /etc/hosts
	echo "${SERVER_IP} ${DOMAIN_NAME}" >> /etc/hosts
}

Set_DNS(){
	sed -i "/^\[main\]/a\dns=none" /etc/NetworkManager/NetworkManager.conf	
	nmcli general reload

	sed -i "/${SERVER_IP}/d" /etc/resolv.conf
	nameserver_line=`grep -n "^nameserver " /etc/resolv.conf | head -n 1 |awk -F: '{print $1}'`
	local line=1 # no search no nameserver
	if [ -z $nameserver_line ]; then
		search_line=`grep -n "^search " /etc/resolv.conf | tail -n 1 |awk -F: '{print $1}'`
		if [ ! -z $search_line ]; then
			line=`expr $search_line + 1`
		fi
	else
		line=${nameserver_line}
	fi
	echo "/etc/resolv.conf: insert line(${line})"
	sed -i "${line}i\nameserver ${SERVER_IP}" /etc/resolv.conf
}

Set_RepRepro() {
	# Register apt repository
	eval $(apt-config shell APT_SOURCE_PARTS Dir::Etc::sourceparts/d)
	CODE_SOURCE_PART=${APT_SOURCE_PARTS}${SERVER_IP}.list

	eval $(apt-config shell APT_TRUSTED_PARTS Dir::Etc::trustedparts/d)
	CODE_TRUSTED_PART=${APT_TRUSTED_PARTS}${SERVER_IP}.gpg

	if [ -f $CODE_SOURCE_PART ]; then
		rm -f $CODE_SOURCE_PART
	fi

	echo "deb [trusted=yes] http://${SERVER_IP}:9778/tiger tiger main" > $repo_source

	if [ -f $CODE_TRUSTED_PART ]; then
		rm -f $CODE_TRUSTED_PART
	fi
	wget -qO - http://${SERVER_IP}:9778/nfs-domain.gpg.key | apt-key add -
	apt-get update -y
}

install_chrony() {
	apt-get install chrony -y
	if [ $? -ne 0 ]; then
		dpkg -i --force-conflicts /usr/share/nfs-domain-client/chrony_4.0-8_amd64.deb
		echo "install chrony result: $?"
		apt remove systemd-timesyncd
	fi
}

Install_Deb_Pack(){
	#if [ ! -f $repo_source ]; then
		#Set_RepRepro
	#fi

	# 非交互式执行脚本用到
	#apt-get install expect  -y

	#apt-get install ntpdate  -y
	#ntpdate -s ${SERVER_IP} &

	# 命令工具，非必须; 确保pam配置中pam_sss.so能重装
	#apt-get install sssd-tools -y
	#apt-get install ldb-tools -y

	install_chrony
	var=`cat /etc/chrony/chrony.conf | grep "pool" | wc -l`
	if [ $var -ne 0 ]; then  
		sed -i "/^pool / s/^\(.*\)$/#\1/g" /etc/chrony/chrony.conf
	fi
	sed -i "/${DOMAIN_NAME}/d" /etc/chrony/chrony.conf

	vars=`cat /etc/chrony/chrony.conf | grep $DOMAIN_NAME | wc -l`
	if [ $vars -eq 0 ]; then
		sed -i "/^#pool /a\server $DOMAIN_NAME iburst" /etc/chrony/chrony.conf
	fi

	systemctl enable chrony
	systemctl restart chrony

	#准备加域，启用NS中sss, 启用sssd服务
	if [ ! -f "/etc/sssd/sssd.conf" ]; then
		echo "enable sssd.service"
		systemctl enable sssd.service
		systemctl start sssd.service

		num=`grep -c "#sudoers:.*sss" /etc/nsswitch.conf `
		if [ $num -ne 0 ]; then  
			echo "enable sudoers: sss in nsswitch.conf"
			sed -i "s/^#sudoers/sudoers/" /etc/nsswitch.conf
		fi
	fi

	# 更新config.ini配置ip_port字段
	num=`grep -c "^add_domain_success=true" $domain_config`
	if [ $num -eq 0 ]; then
		# 更新ip_port
		ip_port_line=`grep -n "^ip_port=" $domain_config |awk -F: '{print $1}'`
		sed -i "${ip_port_line}"'d' $domain_config
		sed -i "${ip_port_line}i\ip_port=$SERVER_IP:8082" $domain_config

		# 更新domain_name
		domain_name_line=`grep -n "^domain_name=" $domain_config | awk -F: '{print $1}'`
		if [ -n "$domain_name_line" ]; then
			sed -i "${domain_name_line}"'d' $domain_config
		else
			domain_name_line=`expr $ip_port_line + 1`
		fi
		sed -i "${domain_name_line}i\domain_name=$DOMAIN_NAME" $domain_config
	fi
}

startTime=`date +%s`
Set_DNS
Set_HOSTS
Install_Deb_Pack
endTime=`date +%s`
((outTime=$endTime-$startTime))
echo -e "Time consumed:\033[32m $outTime \033[0mSeconds!"

