#! /bin/bash

# 本地调试打印开关
Debug=1

# 磁盘剩余空间最低值
Disk_space_limit=1000

# 发送通知
send_notify () {
	# 设置通知标题和内容
	title=$1
	message=$2

	# 调用 notify-send 命令发送通知
	notify-send "$title" "$message"
}

# 检查网络连接
check_network () {
	# 设置要测试的 IP 地址
	ip_address=$1

	# 发送一个 ping 请求，等待 1 秒钟
	ping -c 1 -W 1 $ip_address > /dev/null

	# 检查返回值，如果返回值为 0，则连接正常，否则连接失败
	if [ $? -eq 0 ]
	then
	  	[ $Debug -gt 0 ] && echo "网络检查" "正常连接服务器 $ip_address"
	else
	  	send_notify "网络检查" "连接服务器地址$ip_address失败！"
	fi
}

# 检查磁盘空间
check_disk () {
	# 设置要检查的挂载点
	mount_point="/"

	# 获取本地磁盘空间信息，然后解析出可用空间和总空间
	df_output=$(df -hm $mount_point)
	total_space=$(echo "$df_output" | awk 'NR==2 {print $2}')
	available_space=$(echo "$df_output" | awk 'NR==2 {print $4}')

	# 打印可用空间和总空间信息
	if [ $available_space -lt $Disk_space_limit ]
	then
		send_notify "磁盘检查" "剩余磁盘空间不足!"
	else
		[ $Debug -gt 0 ] && echo "Available space on $mount_point: $available_space"
	fi
}

# 检查系统服务
check_service () {
	# 设置要检查的 systemd 服务名
	service_names=("sssd" "chrony" "chronyd" "salt-minion")

	# 检查 systemd 服务状态，如果服务状态为 running，则服务已启动，否则服务未启动
	for service in "${service_names[@]}"
	do 
		status=`systemctl status $service | grep -c "Active: active (running)"`
		if [ $status -eq 1 ]
		then
		  	[ $Debug -gt 0 ] && echo "$service is running"
		else
		  	send_notify "域服务检查" "$service服务未启动"
		fi
	done
}

# 加域前检查
# 参数 服务器IP
pre_add_domain () {
	check_network $1
	check_disk	
	check_service
}

pre_add_domain $1


